Module double_checked

Module double_checked 

Double-Checked Lock Executor

Provides a double-checked lock executor for executing tasks with condition checking and prepare lifecycle support.

Author

Haixing Hu

Re-exports

pub use double_checked_lock_executor::DoubleCheckedLockExecutor;
pub use execution_context::ExecutionContext;
pub use execution_logger::ExecutionLogger;
pub use execution_result::ExecutionResult;
pub use executor_builder::ExecutorBuilder;
pub use executor_config::ExecutorConfig;
pub use executor_error::ExecutorError;
pub use executor_lock_builder::ExecutorLockBuilder;
pub use executor_ready_builder::ExecutorReadyBuilder;

Modules

double_checked_lock_executor
Double-Checked Lock Executor
execution_context
Execution Context
execution_logger
Execution Logger
execution_result
Execution Result
executor_builder
Initial builder for super::DoubleCheckedLockExecutor.
executor_config
Executor Configuration
executor_error
Executor Error
executor_lock_builder
Builder state after a lock has been attached for super::DoubleCheckedLockExecutor.
executor_ready_builder
Ready builder for super::DoubleCheckedLockExecutor (tester set, optional prepare hooks).