pub enum ExecutorError<E>where
E: Display,{
TaskFailed(E),
PrepareFailed(String),
PrepareCommitFailed(String),
PrepareRollbackFailed {
original: String,
rollback: String,
},
LockPoisoned(String),
}Expand description
Executor error types
Defines various error conditions that can occur during executor operation, including task execution errors, prepare failures, prepare commit failures, and prepare rollback failures.
Type Parameters
E- The original error type from task execution
Examples
use qubit_lock::double_checked::ExecutorError;
let error: ExecutorError<String> =
ExecutorError::TaskFailed("task failed".to_string());
println!("Error: {}", error);
let error_with_msg: ExecutorError<String> =
ExecutorError::PrepareFailed("Service is not running".to_string());
println!("Error: {}", error_with_msg);Author
Haixing Hu
Variants
TaskFailed(E)
Task execution failed with original error
PrepareFailed(String)
Preparation action failed
PrepareCommitFailed(String)
Commit action for a successfully completed prepare action failed.
PrepareRollbackFailed
Rollback action for a successfully completed prepare action failed.
Fields
LockPoisoned(String)
Lock poisoned error