pub struct ExecutorLockBuilder<L, T> { /* private fields */ }Expand description
Builder state after a lock has been attached.
Call Self::when to configure the required condition tester.
Type Parameters
L- The lock type implementingLock<T>.T- The data type protected by the lock.
Author
Haixing Hu
Implementations
Sourceimpl<L, T> ExecutorLockBuilder<L, T>where
L: Lock<T>,
impl<L, T> ExecutorLockBuilder<L, T>where
L: Lock<T>,
Sourcepub fn log_unmet_condition(
self,
level: Level,
message: impl Into<String>,
) -> Self
pub fn log_unmet_condition( self, level: Level, message: impl Into<String>, ) -> Self
Configures logging when the double-checked condition is not met.
Sourcepub fn log_prepare_failure(
self,
level: Level,
message_prefix: impl Into<String>,
) -> Self
pub fn log_prepare_failure( self, level: Level, message_prefix: impl Into<String>, ) -> Self
Configures logging when the prepare action fails.
Sourcepub fn log_prepare_commit_failure(
self,
level: Level,
message_prefix: impl Into<String>,
) -> Self
pub fn log_prepare_commit_failure( self, level: Level, message_prefix: impl Into<String>, ) -> Self
Configures logging when the prepare commit action fails.
Sourcepub fn log_prepare_rollback_failure(
self,
level: Level,
message_prefix: impl Into<String>,
) -> Self
pub fn log_prepare_rollback_failure( self, level: Level, message_prefix: impl Into<String>, ) -> Self
Configures logging when the prepare rollback action fails.
Sourcepub fn when<Tst>(self, tester: Tst) -> ExecutorReadyBuilder<L, T>
pub fn when<Tst>(self, tester: Tst) -> ExecutorReadyBuilder<L, T>
Configures the required double-checked condition.
The tester is executed outside and inside the lock. State read by the outside check must be safe to access without this executor’s lock.
Parameters
tester- The reusable condition tester.
Returns
The builder state that can configure prepare callbacks and build the executor.