TryLockError

Enum TryLockError 

pub enum TryLockError {
    WouldBlock,
    Poisoned,
}
Non-blocking lock acquisition error.

This error type is used by try_read and try_write to distinguish lock contention from poisoned lock states.

Variants



WouldBlock

The lock is currently held by another thread.



Poisoned

The lock is poisoned due to a panic while the lock was held.