ArcAsyncMutex

Struct ArcAsyncMutex 

pub struct ArcAsyncMutex<T> { /* private fields */ }
Asynchronous Mutex Wrapper

Provides an encapsulation of asynchronous mutex for protecting shared data in asynchronous environments. Supports safe access and modification of shared data across multiple asynchronous tasks.

Features

  • Asynchronously acquires locks, does not block threads
  • Supports trying to acquire locks (non-blocking)
  • Thread-safe, supports multi-threaded sharing
  • Automatic lock management through RAII ensures proper lock release

Usage Example

use qubit_lock::lock::{ArcAsyncMutex, AsyncLock};

let rt = tokio::runtime::Builder::new_current_thread()
    .enable_all()
    .build()
    .unwrap();
rt.block_on(async {
    let counter = ArcAsyncMutex::new(0);

    // Asynchronously modify data
    counter.write(|c| {
        *c += 1;
        println!("Counter: {}", *c);
    }).await;

    // Try to acquire lock
    if let Ok(value) = counter.try_read(|c| *c) {
        println!("Current value: {}", value);
    }
});

Implementations

Source

impl<T> ArcAsyncMutex<T>

pub fn new(data: T) -> Self

Creates a new asynchronous mutex lock

Arguments
  • data - The data to be protected
Returns

Returns a new ArcAsyncMutex instance

Example
use qubit_lock::lock::ArcAsyncMutex;

let lock = ArcAsyncMutex::new(42);