ArcTokioMonitor

Struct ArcTokioMonitor 

pub struct ArcTokioMonitor<T> { /* private fields */ }
Cloneable handle around a TokioMonitor.

Implementations

Source

impl<T> ArcTokioMonitor<T>

pub fn new(state: T) -> Self

Creates an Arc-wrapped Tokio monitor.

Arguments
  • state - Initial protected state.
Returns

A cloneable Tokio monitor handle.

pub async fn async_read<R, F>(&self, f: F) -> R
where F: FnOnce(&T) -> R,

Reads protected state asynchronously.

pub async fn async_write<R, F>(&self, f: F) -> R
where F: FnOnce(&mut T) -> R,

Mutates protected state asynchronously without notifying.

pub async fn async_write_notify_one<R, F>(&self, f: F) -> R
where F: FnOnce(&mut T) -> R,

Mutates protected state asynchronously and wakes one waiter.

pub async fn async_write_notify_all<R, F>(&self, f: F) -> R
where F: FnOnce(&mut T) -> R,

Mutates protected state asynchronously and wakes all waiters.

pub fn notify_one(&self)

Wakes one async waiter.

pub fn notify_all(&self)

Wakes all async waiters.

Methods from Deref<Target = TokioMonitor<T>>

pub async fn async_read<R, F>(&self, f: F) -> R
where F: FnOnce(&T) -> R,

Acquires the monitor and reads the protected state.

Arguments
  • f - Closure that receives an immutable reference to the state.
Returns

The value returned by the closure.

pub async fn async_write<R, F>(&self, f: F) -> R
where F: FnOnce(&mut T) -> R,

Acquires the monitor and mutates the protected state.

This does not notify waiters automatically.

Arguments
  • f - Closure that receives a mutable reference to the state.
Returns

The value returned by the closure.

pub async fn async_write_notify_one<R, F>(&self, f: F) -> R
where F: FnOnce(&mut T) -> R,

Mutates the protected state and wakes one waiter.

Arguments
  • f - Closure that receives a mutable reference to the state.
Returns

The value returned by the closure.

pub async fn async_write_notify_all<R, F>(&self, f: F) -> R
where F: FnOnce(&mut T) -> R,

Mutates the protected state and wakes all waiters.

Arguments
  • f - Closure that receives a mutable reference to the state.
Returns

The value returned by the closure.

pub fn notify_one(&self)

Wakes one async waiter.

pub fn notify_all(&self)

Wakes all async waiters.