AsyncTimeoutNotificationWaiter

Trait AsyncTimeoutNotificationWaiter 

pub trait AsyncTimeoutNotificationWaiter {
    // Required method
    fn async_wait_for<'a>(
        &'a self,
        timeout: Duration,
    ) -> AsyncMonitorFuture<'a, WaitTimeoutStatus>;
}
Waits asynchronously for a notification with a relative timeout.

Required Methods

fn async_wait_for<'a>( &'a self, timeout: Duration, ) -> AsyncMonitorFuture<'a, WaitTimeoutStatus>

Returns a future that resolves after notification or timeout.

Arguments
  • timeout - Maximum relative duration to wait, measured from this method call.
Returns

A future resolving to the timeout status.

Implementors

Source

impl<T: Send> AsyncTimeoutNotificationWaiter for ArcMockMonitor<T>

Available on crate feature async only.
Source

impl<T: Send> AsyncTimeoutNotificationWaiter for ArcTokioMonitor<T>

Source

impl<T: Send> AsyncTimeoutNotificationWaiter for MockMonitor<T>

Available on crate feature async only.
Source

impl<T: Send> AsyncTimeoutNotificationWaiter for TokioMonitor<T>