pub trait Notifier {
// Required methods
fn notify_one(&self);
fn notify_all(&self);
}Expand description
Sends notification signals to waiters.
Notifications are coordination signals. They do not carry state by themselves, so condition waiters should always recheck the protected state after waking.
Required Methods
Sourcefn notify_one(&self)
fn notify_one(&self)
Wakes one waiter if one is currently blocked.
Sourcefn notify_all(&self)
fn notify_all(&self)
Wakes all current waiters.