TimeoutNotificationWaiter

Trait TimeoutNotificationWaiter 

pub trait TimeoutNotificationWaiter {
    // Required method
    fn wait_for(&self, timeout: Duration) -> WaitTimeoutStatus;
}
Waits for a notification with a relative timeout.

Required Methods

fn wait_for(&self, timeout: Duration) -> WaitTimeoutStatus

Blocks until a notification is observed or the timeout expires.

Arguments
  • timeout - Maximum relative duration to wait.
Returns

A status describing whether the wait returned before the timeout.

Implementors