pub struct ArcStdMutex<T> { /* private fields */ }Expand description
Synchronous Standard Mutex Wrapper
Provides an encapsulation of synchronous mutex using std::sync::Mutex for protecting shared data in synchronous environments. Supports safe access and modification of shared data across multiple threads.
Features
- Synchronously acquires locks, may block threads
- Supports trying to acquire locks (non-blocking)
- Thread-safe, supports multi-threaded sharing
- Automatic lock management through RAII ensures proper lock release
- Implements
DerefandAsRefto expose the underlyingstd::sync::MutexAPI when guard-based access is needed
Usage Example
use qubit_lock::lock::{ArcStdMutex, Lock};
let counter = ArcStdMutex::new(0);
// Synchronously modify data
counter.write(|c| {
*c += 1;
println!("Counter: {}", *c);
});
// Try to acquire lock
if let Ok(value) = counter.try_read(|c| *c) {
println!("Current value: {}", value);
}Implementations
Methods from Deref<Target = Mutex<T>>
Sourcepub fn get_cloned(&self) -> Result<T, PoisonError<()>>where
T: Clone,
🔬This is a nightly-only experimental API. (lock_value_accessors)
pub fn get_cloned(&self) -> Result<T, PoisonError<()>>where
T: Clone,
lock_value_accessors)Returns the contained value by cloning it.
Errors
If another user of this mutex panicked while holding the mutex, then this call will return an error instead.
Examples
#![feature(lock_value_accessors)]
use std::sync::Mutex;
let mut mutex = Mutex::new(7);
assert_eq!(mutex.get_cloned().unwrap(), 7);Sourcepub fn set(&self, value: T) -> Result<(), PoisonError<T>>
🔬This is a nightly-only experimental API. (lock_value_accessors)
pub fn set(&self, value: T) -> Result<(), PoisonError<T>>
lock_value_accessors)Sets the contained value.
Errors
If another user of this mutex panicked while holding the mutex, then
this call will return an error containing the provided value instead.
Examples
#![feature(lock_value_accessors)]
use std::sync::Mutex;
let mut mutex = Mutex::new(7);
assert_eq!(mutex.get_cloned().unwrap(), 7);
mutex.set(11).unwrap();
assert_eq!(mutex.get_cloned().unwrap(), 11);Sourcepub fn replace(&self, value: T) -> Result<T, PoisonError<T>>
🔬This is a nightly-only experimental API. (lock_value_accessors)
pub fn replace(&self, value: T) -> Result<T, PoisonError<T>>
lock_value_accessors)Replaces the contained value with value, and returns the old contained value.
Errors
If another user of this mutex panicked while holding the mutex, then
this call will return an error containing the provided value instead.
Examples
#![feature(lock_value_accessors)]
use std::sync::Mutex;
let mut mutex = Mutex::new(7);
assert_eq!(mutex.replace(11).unwrap(), 7);
assert_eq!(mutex.get_cloned().unwrap(), 11);1.0.0 · Sourcepub fn lock(&self) -> Result<MutexGuard<'_, T>, PoisonError<MutexGuard<'_, T>>>
pub fn lock(&self) -> Result<MutexGuard<'_, T>, PoisonError<MutexGuard<'_, T>>>
Acquires a mutex, blocking the current thread until it is able to do so.
This function will block the local thread until it is available to acquire the mutex. Upon returning, the thread is the only thread with the lock held. An RAII guard is returned to allow scoped unlock of the lock. When the guard goes out of scope, the mutex will be unlocked.
The exact behavior on locking a mutex in the thread which already holds the lock is left unspecified. However, this function will not return on the second call (it might panic or deadlock, for example).
Errors
If another user of this mutex panicked while holding the mutex, then this call will return an error once the mutex is acquired. The acquired mutex guard will be contained in the returned error.
Panics
This function might panic when called if the lock is already held by the current thread.
Examples
use std::sync::{Arc, Mutex};
use std::thread;
let mutex = Arc::new(Mutex::new(0));
let c_mutex = Arc::clone(&mutex);
thread::spawn(move || {
*c_mutex.lock().unwrap() = 10;
}).join().expect("thread::spawn failed");
assert_eq!(*mutex.lock().unwrap(), 10);1.0.0 · Sourcepub fn try_lock(
&self,
) -> Result<MutexGuard<'_, T>, TryLockError<MutexGuard<'_, T>>>
pub fn try_lock( &self, ) -> Result<MutexGuard<'_, T>, TryLockError<MutexGuard<'_, T>>>
Attempts to acquire this lock.
If the lock could not be acquired at this time, then Err is returned.
Otherwise, an RAII guard is returned. The lock will be unlocked when the
guard is dropped.
This function does not block.
Errors
If another user of this mutex panicked while holding the mutex, then
this call will return the Poisoned error if the mutex would
otherwise be acquired. An acquired lock guard will be contained
in the returned error.
If the mutex could not be acquired because it is already locked, then
this call will return the WouldBlock error.
Examples
use std::sync::{Arc, Mutex};
use std::thread;
let mutex = Arc::new(Mutex::new(0));
let c_mutex = Arc::clone(&mutex);
thread::spawn(move || {
let mut lock = c_mutex.try_lock();
if let Ok(ref mut mutex) = lock {
**mutex = 10;
} else {
println!("try_lock failed");
}
}).join().expect("thread::spawn failed");
assert_eq!(*mutex.lock().unwrap(), 10);1.2.0 · Sourcepub fn is_poisoned(&self) -> bool
pub fn is_poisoned(&self) -> bool
Determines whether the mutex is poisoned.
If another thread is active, the mutex can still become poisoned at any
time. You should not trust a false value for program correctness
without additional synchronization.
Examples
use std::sync::{Arc, Mutex};
use std::thread;
let mutex = Arc::new(Mutex::new(0));
let c_mutex = Arc::clone(&mutex);
let _ = thread::spawn(move || {
let _lock = c_mutex.lock().unwrap();
panic!(); // the mutex gets poisoned
}).join();
assert_eq!(mutex.is_poisoned(), true);1.77.0 · Sourcepub fn clear_poison(&self)
pub fn clear_poison(&self)
Clear the poisoned state from a mutex.
If the mutex is poisoned, it will remain poisoned until this function is called. This allows recovering from a poisoned state and marking that it has recovered. For example, if the value is overwritten by a known-good value, then the mutex can be marked as un-poisoned. Or possibly, the value could be inspected to determine if it is in a consistent state, and if so the poison is removed.
Examples
use std::sync::{Arc, Mutex};
use std::thread;
let mutex = Arc::new(Mutex::new(0));
let c_mutex = Arc::clone(&mutex);
let _ = thread::spawn(move || {
let _lock = c_mutex.lock().unwrap();
panic!(); // the mutex gets poisoned
}).join();
assert_eq!(mutex.is_poisoned(), true);
let x = mutex.lock().unwrap_or_else(|mut e| {
**e.get_mut() = 1;
mutex.clear_poison();
e.into_inner()
});
assert_eq!(mutex.is_poisoned(), false);
assert_eq!(*x, 1);Sourcepub fn data_ptr(&self) -> *mut T
🔬This is a nightly-only experimental API. (mutex_data_ptr)
pub fn data_ptr(&self) -> *mut T
mutex_data_ptr)Returns a raw pointer to the underlying data.
The returned pointer is always non-null and properly aligned, but it is the user’s responsibility to ensure that any reads and writes through it are properly synchronized to avoid data races, and that it is not read or written through after the mutex is dropped.