pub struct MockMonitor<T> { /* private fields */ }Expand description
Monitor implementation for deterministic tests.
MockMonitor protects a state value like the real monitor implementations,
but timeout methods use manually controlled mock elapsed time. Advancing the
mock time wakes waiters so they can recheck predicates and timeout budgets.
Implementations
Sourceimpl<T> MockMonitor<T>
impl<T> MockMonitor<T>
Sourcepub fn set_elapsed(&self, elapsed: Duration)
pub fn set_elapsed(&self, elapsed: Duration)
Sets the current mock elapsed time.
This wakes timeout waiters so they can recheck their budgets.
Arguments
elapsed- New mock elapsed time.
Sourcepub fn advance(&self, duration: Duration)
pub fn advance(&self, duration: Duration)
Advances mock elapsed time by a relative duration.
Arguments
duration- Duration added to the current mock elapsed time.
Sourcepub fn reset_elapsed(&self)
pub fn reset_elapsed(&self)
Resets mock elapsed time to zero.
Sourcepub fn write_notify_one<R, F>(&self, f: F) -> R
pub fn write_notify_one<R, F>(&self, f: F) -> R
Sourcepub fn write_notify_all<R, F>(&self, f: F) -> R
pub fn write_notify_all<R, F>(&self, f: F) -> R
Sourcepub fn notify_one(&self)
pub fn notify_one(&self)
Wakes one waiter if one is blocked.
Sourcepub fn notify_all(&self)
pub fn notify_all(&self)
Wakes all waiters.