Notifier

Trait Notifier 

pub trait Notifier {
    // Required methods
    fn notify_one(&self);
    fn notify_all(&self);
}
Sends notification signals to waiters.

Notifications are coordination signals. They do not carry state by themselves, so condition waiters should always recheck the protected state after waking.

Required Methods

fn notify_one(&self)

Wakes one waiter if one is currently blocked.

fn notify_all(&self)

Wakes all current waiters.

Implementors